from django.shortcuts import render, get_object_or_404, redirect
from django.http import HttpResponse
from django.contrib.auth.decorators import login_required
from django.contrib.auth import views as auth_views

from .models import Transaction, TransactionForm, Invoice, InvoiceForm, Category


@login_required
def index(request):
    _transactions = Transaction.objects.order_by("-date")[:5]
    _categories = Category.objects.filter(parent=None)

    def _cat_list(cat):
        children = []
        for child in Category.objects.filter(parent=cat):
            children += _cat_list(child)
        if len(children) == 0:
            return (cat.name,)
        return cat.name, children

    _cats = []
    for cat in _categories:
        _cats += _cat_list(cat)

    context = {
        "transactions": _transactions,
        "categories": _cats,
    }
    return render(request, "main/index.html", context)


class LoginView(auth_views.LoginView):
    template_name = "main/login.html"
    next_page = "index"


class LogoutView(auth_views.LogoutView):
    next_page = "login"


@login_required
def transaction(request, uuid=None):
    if uuid is None:
        _transaction = Transaction()
        _invoices = []
    else:
        _transaction = get_object_or_404(Transaction, id=uuid)
        _invoices = Invoice.objects.filter(transaction=_transaction)
    return render(
        request,
        "main/transaction.html",
        {
            "transaction": _transaction,
            "form": TransactionForm(instance=_transaction),
            "invoices": _invoices,
            "invoice_form": InvoiceForm(instance=Invoice(transaction=_transaction)),
        },
    )


@login_required
def update_transaction(request, uuid):
    try:
        _transaction = Transaction.objects.get(id=uuid)
    except Transaction.DoesNotExist:
        _transaction = Transaction(id=uuid)
    _form = TransactionForm(request.POST, instance=_transaction)
    _form.save()
    return redirect(transaction, uuid=uuid)


@login_required
def invoice(request, uuid):
    _invoice = get_object_or_404(Invoice, id=uuid)
    with _invoice.file.open() as _file:
        return HttpResponse(_file.read(), content_type="application/pdf")


@login_required
def add_invoice(request, uuid):
    _transaction = get_object_or_404(Transaction, id=uuid)
    _invoice = Invoice(transaction=_transaction)
    _form = InvoiceForm(request.POST, request.FILES, instance=_invoice)
    _form.save()
    return redirect(transaction, uuid=uuid)


@login_required
def del_invoice(request, uuid, invoice_id):
    _invoice = get_object_or_404(Invoice, id=invoice_id)
    _invoice.delete()
    return redirect(transaction, uuid=uuid)
